#!/usr/bin/perl -wT

# @(#) Generate a MP3 playlist from directories
# @(#) iekko <iekko@free.fr>
# @(#) $Id: dirxspf.pl,v 1.8 2011-01-16 16:30:03 iekko Exp $

# --- Modules

use strict;
use warnings;
use File::Basename qw(basename);
use Getopt::Std;
use MP3::Info qw(get_mp3info get_mp3tag);
use URI::URL;
use Encode qw(encode);

# --- Configuration

# Environment
$ENV{PATH} = '/bin:/usr/bin:/usr/local/bin';
$Getopt::Std::STANDARD_HELP_VERSION = 1;
my $NAME = basename($0);
my $VERSION = '0.1';
my $AUTHOR = 'iekko <iekko@free.fr>';
my $ACTION = 'Generate a MP3 playlist from directories';
my $ENCODING = 'UTF-8';

# Header
my $HEADER = <<EOF;
<?xml version="1.0" encoding="$ENCODING" ?>
<playlist version="1" xmlns="http://xspf.org/ns/0/">
	<trackList>
EOF

# Footer
my $FOOTER = <<EOF;
	</trackList>
</playlist>
EOF

# Default
my @IN_DIRS = ($ENV{PWD});
my $CUT = '';
my $PREFIX = '';
my $TYPE_RE = '^(unix|dos|url)$';
my $TYPE = 'unix';


# --- Fonctions

# Show version
sub VERSION_MESSAGE {
	print <<EOM;
$NAME $VERSION
Written by $AUTHOR.
EOM
}

# Show help
sub HELP_MESSAGE {
	print <<EOM;
Usage: $NAME [OPTION...] [DIRECTORY...]
$ACTION.
Options:
  --help         display this help
  --version      display version information
  --cut, -c      cut a prefix from a location
  --prefix, -p   add a prefix to location
  --type, -t     apply a type to location
EOM
}

# Add to a list the MP3 files found in a directory
sub add_mp3($$) {
	my ($list, $dir) = @_;
	my @files;
	my $file;

	opendir DIR, $dir or die "$NAME: $dir: can't open directory\n";
	@files = readdir DIR;
	closedir DIR;

	foreach (@files) {
		next if $_ =~ /^\.\.?$/;
		$file = "$dir/$_";
		if (-d $file) {
			&add_mp3($list, $file);
		} elsif ($file =~ /\.mp3$/i) {
			push @{$list}, $file;
		}
	}
}

# Get extended informations from a MP3 file
sub get_extinfo($$$$) {
	my ($file, $cut, $prefix, $type) = @_;
	my $extinfo = "\t\t<track>\n";
	my $info = get_mp3info($file);
	my $tag = get_mp3tag($file);
	my $artist = $tag->{ARTIST};
	my $title = $tag->{TITLE};

	$extinfo .= "\t\t\t<location><![CDATA["
		. apply_type($type, $prefix . cut_prefix($cut, $file))
		. "]]></location>\n";
	if (defined $info) {
		if ($artist eq '' || $title eq '') {
			warn "$NAME: $file: can't read extended informations from file\n";
			$extinfo .= "\t\t\t<title><![CDATA["
				. basename($file, '.mp3', '.mP3', '.Mp3', '.MP3')
				. "]]></title>\n";
		} else {
			$extinfo .= "\t\t\t<creator><![CDATA[$artist]]></creator>\n"
				. "\t\t\t<title><![CDATA[$title]]></title>\n";
		}
		$extinfo .= "\t\t\t<duration>"
			. int($info->{SECS}) * 1000
			. "</duration>\n";
	} else {
		warn "$NAME: $file: can't read informations from file\n";
		$extinfo .= "\t\t\t<title><![CDATA["
			. basename($file, '.mp3', '.mP3', '.Mp3', '.MP3')
			. "]]></title>\n";
	}

	return "$extinfo\t\t</track>\n";;
}


# Cut a prefix from a location
sub cut_prefix($$) {
	my ($prefix, $location) = @_;

	$location =~ s/^\Q$prefix\E//;

	return $location;
}

# Apply a type to a location
sub apply_type($$) {
	my ($type, $location) = @_;

	if ($type eq 'unix') {
		$location =~ s,\\,/,g;
	} elsif ($type eq 'dos') {
		$location =~ s,/,\\,g;
	} elsif ($type eq 'url') {
		$location =~ s,\\,/,g;
		$location = URI::URL->new($location)->as_string;
	}

	return $location;
}


# --- Options

my %options;

# Get
getopts 'c:p:t:', \%options
	or die "$NAME: invalid option\n";
$PREFIX = $options{'p'} if defined $options{'p'};
$CUT = $options{'c'} if defined $options{'c'};
if (defined $options{'t'}) {
	if ($options{'t'} =~ /$TYPE_RE/) {
		$TYPE = $options{'t'},
	} else {
		die "$NAME: $options{'t'}: invalid type\n";
	}
}
@IN_DIRS = @ARGV if $#ARGV != -1;

# Check
foreach (@IN_DIRS) {
	die "$NAME: $_: can't find directory\n" unless -d;
	die "$NAME: $_: can't read directory\n" unless -r;
	die "$NAME: $_: can't open directory\n" unless -x;
}


# --- Main

my @files;

# Add MP3 files from directories
add_mp3(\@files, $_) foreach @IN_DIRS;

# Print MP3 playlist
print $HEADER;
print encode($ENCODING, get_extinfo($_, $CUT, $PREFIX, $TYPE)) foreach sort @files;
print $FOOTER, "\n";

